# Tempered Particle Filter Documentation

## Introduction
- The tempered particle filter is a particle filtering method which can approximate the log-likelihood value
implied by a general (potentially non-linear) state space system. The filter was introduced by Ed Herbst and
Frank Schorfheide in a recent paper (cited in the [README](https://github.com/FRBNY-DSGE/StateSpaceRoutines.jl/tree/doc)
in the root directory of the StateSpaceRoutines repository).
- An example of the tempered particle filter can be found [here](https://github.com/FRBNY-DSGE/StateSpaceRoutines.jl/blob/master/docs/examples/tempered_particle_filter/tempered_particle_filter_ex.jl).

## Inputs to the function
- The inputs for `tempered_particle_filter` are as follows:
    + The transition and measurement equations: `Φ`,and `Ψ`, which are instances of the `Function` type
    + The shock and measurement error distributions: `F_ϵ`, `F_u`, which are instances of the `Distribution` type
    + A matrix of initial draws (number of states by number of particles): `s_init`.
    These draws can be chosen by the user or can be generated by using the `initialize_state_draws` function,
    which iterates the states forward using `Φ` and `F_ϵ` and generates draws after accounting for an initial
    burn-in period. The sample is also thinned to prevent serial correlation.
    + The tuning hyperparameters (like the number of particles) are set by default but can be specified by
    keyword arguments in the function call. Detailed documentation of these can be found by reading the
    docstring, i.e. calling `?tempered_particle_filter` in the Julia REPL after loading the StateSpaceRoutines
    package.

## General State Space System
```
s_{t+1} = Φ(s_t, ϵ_t)        (transition equation)
y_t     = Ψ(s_t, u_t)        (measurement equation)

ϵ_t ∼ F_ϵ(∙; θ)
u_t ∼ F_u(∙; θ), where F_u is N(0, HH), where HH is the variance matrix of the i.i.d measurement error
Cov(ϵ_t, u_t) = 0
```
